@@ -8,7 +8,7 @@ Alternatively, sound files can be opened as :class:`SoundFile` objects.
 For further information, see http://pysoundfile.readthedocs.org/.
 
 """
-__version__ = "0.8.1"
+__version__ = "0.9.0"
 
 import os as _os
 import sys as _sys
@@ -31,6 +31,7 @@ enum
 
 enum
 {
+    SFC_GET_LIB_VERSION             = 0x1000,
     SFC_GET_LOG_INFO                = 0x1001,
     SFC_GET_FORMAT_INFO             = 0x1028,
 
@@ -92,10 +93,12 @@ sf_count_t  sf_read_int      (SNDFILE *sndfile, int *ptr, sf_count_t items) ;
 sf_count_t  sf_read_float    (SNDFILE *sndfile, float *ptr, sf_count_t items) ;
 sf_count_t  sf_read_double   (SNDFILE *sndfile, double *ptr, sf_count_t items) ;
 
-sf_count_t  sf_readf_short   (SNDFILE *sndfile, short *ptr, sf_count_t frames) ;
-sf_count_t  sf_readf_int     (SNDFILE *sndfile, int *ptr, sf_count_t frames) ;
-sf_count_t  sf_readf_float   (SNDFILE *sndfile, float *ptr, sf_count_t frames) ;
-sf_count_t  sf_readf_double  (SNDFILE *sndfile, double *ptr, sf_count_t frames) ;
+/* Note: Data ptr argument types are declared as void* here in order to
+         avoid an implicit cast warning. (gh183). */
+sf_count_t  sf_readf_short   (SNDFILE *sndfile, void *ptr, sf_count_t frames) ;
+sf_count_t  sf_readf_int     (SNDFILE *sndfile, void *ptr, sf_count_t frames) ;
+sf_count_t  sf_readf_float   (SNDFILE *sndfile, void *ptr, sf_count_t frames) ;
+sf_count_t  sf_readf_double  (SNDFILE *sndfile, void *ptr, sf_count_t frames) ;
 
 sf_count_t  sf_write_short   (SNDFILE *sndfile, short *ptr, sf_count_t items) ;
 sf_count_t  sf_write_int     (SNDFILE *sndfile, int *ptr, sf_count_t items) ;
@@ -114,6 +117,7 @@ sf_count_t  sf_write_raw     (SNDFILE *sndfile, void *ptr, sf_count_t bytes) ;
 
 const char* sf_get_string    (SNDFILE *sndfile, int str_type) ;
 int         sf_set_string    (SNDFILE *sndfile, int str_type, const char* str) ;
+const char * sf_version_string (void) ;
 
 typedef sf_count_t  (*sf_vio_get_filelen) (void *user_data) ;
 typedef sf_count_t  (*sf_vio_seek)        (sf_count_t offset, int whence, void *user_data) ;
@@ -210,6 +214,10 @@ _subtypes = {
     'DPCM_8':    0x0050,  # 8 bit differential PCM (XI only)
     'DPCM_16':   0x0051,  # 16 bit differential PCM (XI only)
     'VORBIS':    0x0060,  # Xiph Vorbis encoding.
+    'ALAC_16':   0x0070,  # Apple Lossless Audio Codec (16 bit).
+    'ALAC_20':   0x0071,  # Apple Lossless Audio Codec (20 bit).
+    'ALAC_24':   0x0072,  # Apple Lossless Audio Codec (24 bit).
+    'ALAC_32':   0x0073,  # Apple Lossless Audio Codec (32 bit).
 }
 
 _endians = {
@@ -257,7 +265,7 @@ _ffi_types = {
 
 try:
     _snd = _ffi.dlopen('sndfile')
-except OSError as err:
+except OSError:
     if _sys.platform == 'darwin':
         _libname = 'libsndfile.dylib'
     elif _sys.platform == 'win32':
@@ -269,6 +277,10 @@ except OSError as err:
         _os.path.dirname(_os.path.abspath(__file__)),
         '_soundfile_data', _libname))
 
+__libsndfile_version__ = _ffi.string(_snd.sf_version_string()).decode('utf-8', 'replace')
+if __libsndfile_version__.startswith('libsndfile-'):
+    __libsndfile_version__ = __libsndfile_version__[len('libsndfile-'):]
+
 
 def read(file, frames=-1, start=0, stop=None, dtype='float64', always_2d=False,
          fill_value=None, out=None, samplerate=None, channels=None,
@@ -481,13 +493,15 @@ def blocks(file, blocksize=None, overlap=0, frames=-1, start=0, stop=None,
 
 class _SoundFileInfo(object):
     """Information about a SoundFile"""
+
     def __init__(self, file, verbose):
         self.verbose = verbose
         with SoundFile(file) as f:
             self.name = f.name
             self.samplerate = f.samplerate
             self.channels = f.channels
-            self.duration = len(f)/f.samplerate
+            self.frames = len(f)
+            self.duration = self.frames/f.samplerate
             self.format = f.format
             self.subtype = f.subtype
             self.endian = f.endian
@@ -520,6 +534,7 @@ class _SoundFileInfo(object):
             info += "\n".join(
                 ["\nendian: {0.endian}",
                  "sections: {0.sections}",
+                 "frames: {0.frames}",
                  'extra_info: """',
                  '    {1}"""'])
         indented_extra_info = ("\n"+" "*4).join(self.extra_info.split("\n"))
@@ -616,7 +631,6 @@ def default_subtype(format):
 
 
 class SoundFile(object):
-
     """A sound file.
 
     For more documentation see the __init__() docstring (which is also
@@ -768,7 +782,7 @@ class SoundFile(object):
         info = _ffi.new("char[]", 2**14)
         _snd.sf_command(self._file, _snd.SFC_GET_LOG_INFO,
                         info, _ffi.sizeof(info))
-        return _ffi.string(info).decode()
+        return _ffi.string(info).decode('utf-8', 'replace')
 
     # avoid confusion if something goes wrong before assigning self._file:
     _file = None
@@ -803,7 +817,7 @@ class SoundFile(object):
         if name in _str_types:
             self._check_if_closed()
             data = _snd.sf_get_string(self._file, _str_types[name])
-            return _ffi.string(data).decode() if data else ""
+            return _ffi.string(data).decode('utf-8', 'replace') if data else ""
         else:
             raise AttributeError(
                 "'SoundFile' object has no attribute {0!r}".format(name))
@@ -956,10 +970,10 @@ class SoundFile(object):
                 out[frames:] = fill_value
         return out
 
-    def buffer_read(self, frames=-1, ctype='double'):
+    def buffer_read(self, frames=-1, ctype=None, dtype=None):
         """Read from the file and return data as buffer object.
 
-        Reads the given number of frames in the given data format
+        Reads the given number of `frames` in the given data format
         starting at the current read/write position.  This advances the
         read/write position by the same number of frames.
         By default, all frames from the current read/write position to
@@ -971,8 +985,8 @@ class SoundFile(object):
         frames : int, optional
             The number of frames to read. If `frames < 0`, the whole
             rest of the file is read.
-        ctype : {'double', 'float', 'int', 'short'}, optional
-            Audio data will be converted to the given C data type.
+        dtype : {'float64', 'float32', 'int32', 'int16'}
+            Audio data will be converted to the given data type.
 
         Returns
         -------
@@ -985,30 +999,32 @@ class SoundFile(object):
 
         """
         frames = self._check_frames(frames, fill_value=None)
+        dtype = self._ctype_is_deprecated(ctype, dtype)
+        ctype = self._check_dtype(dtype)
         cdata = _ffi.new(ctype + '[]', frames * self.channels)
         read_frames = self._cdata_io('read', cdata, ctype, frames)
         assert read_frames == frames
         return _ffi.buffer(cdata)
 
-    def buffer_read_into(self, buffer, ctype='double'):
-        """Read from the file into a buffer object.
+    def buffer_read_into(self, buffer, ctype=None, dtype=None):
+        """Read from the file into a given buffer object.
 
-        Reads the given number of frames in the given data format
-        starting at the current read/write position.  This advances the
-        read/write position by the same number of frames.
-        By default, all frames from the current read/write position to
-        the end of the file are returned.
-        Use :meth:`.seek` to move the current read/write position.
+        Fills the given `buffer` with frames in the given data format
+        starting at the current read/write position (which can be
+        changed with :meth:`.seek`) until the buffer is full or the end
+        of the file is reached.  This advances the read/write position
+        by the number of frames that were read.
 
         Parameters
         ----------
-        out : writable buffer, optional
-            If specified, audio data from the file is written to this
-            buffer instead of a newly created buffer.
+        buffer : writable buffer
+            Audio frames from the file are written to this buffer.
+        dtype : {'float64', 'float32', 'int32', 'int16'}
+            The data type of `buffer`.
 
         Returns
         -------
-        num_read : int
+        int
             The number of frames that were read from the file.
             This can be less than the size of `buffer`.
             The rest of the buffer is not filled with meaningful data.
@@ -1018,6 +1034,8 @@ class SoundFile(object):
         buffer_read, .read
 
         """
+        dtype = self._ctype_is_deprecated(ctype, dtype)
+        ctype = self._check_dtype(dtype)
         cdata, frames = self._check_buffer(buffer, ctype)
         frames = self._cdata_io('read', cdata, ctype, frames)
         return frames
@@ -1075,26 +1093,29 @@ class SoundFile(object):
         assert written == len(data)
         self._update_len(written)
 
-    def buffer_write(self, data, ctype):
+    def buffer_write(self, data, ctype=None, dtype=None):
         """Write audio data from a buffer/bytes object to the file.
 
-        Writes a number of frames at the read/write position to the
-        file. This also advances the read/write position by the same
-        number of frames and enlarges the file if necessary.
+        Writes the contents of `data` to the file at the current
+        read/write position.
+        This also advances the read/write position by the number of
+        frames that were written and enlarges the file if necessary.
 
         Parameters
         ----------
         data : buffer or bytes
-            A buffer object or bytes containing the audio data to be
+            A buffer or bytes object containing the audio data to be
             written.
-        ctype : {'double', 'float', 'int', 'short'}, optional
-            The data type of the audio data stored in `buffer`.
+        dtype : {'float64', 'float32', 'int32', 'int16'}
+            The data type of the audio data stored in `data`.
 
         See Also
         --------
         .write, buffer_read
 
         """
+        dtype = self._ctype_is_deprecated(ctype, dtype)
+        ctype = self._check_dtype(dtype)
         cdata, frames = self._check_buffer(data, ctype)
         written = self._cdata_io('write', cdata, ctype, frames)
         assert written == frames
@@ -1332,6 +1353,7 @@ class SoundFile(object):
 
     def _check_buffer(self, data, ctype):
         """Convert buffer to cdata and check for valid size."""
+        assert ctype in _ffi_types.values()
         if not isinstance(data, bytes):
             data = _ffi.from_buffer(data)
         frames, remainder = divmod(len(data),
@@ -1349,6 +1371,31 @@ class SoundFile(object):
             shape = frames,
         return np.empty(shape, dtype, order='C')
 
+    def _check_dtype(self, dtype):
+        """Check if dtype string is valid and return ctype string."""
+        try:
+            return _ffi_types[dtype]
+        except KeyError:
+            raise ValueError("dtype must be one of {0!r}".format(
+                sorted(_ffi_types.keys())))
+
+    def _ctype_is_deprecated(self, ctype, dtype):
+        """Show warning if ctype is used instead of dtype.
+
+        At some point, ctype arguments shall be removed and the
+        corresponding dtype arguments shall lose their default value.
+
+        """
+        if ctype is not None:
+            from warnings import warn
+            warn('ctype is deprecated; use dtype instead', Warning)
+            if dtype is not None:
+                raise TypeError('Use dtype instead of ctype')
+            for k, v in _ffi_types.items():
+                if v == ctype:
+                    return k
+        return dtype
+
     def _array_io(self, action, array, frames):
         """Check array and call low-level IO function."""
         if (array.ndim not in (1, 2) or
@@ -1357,19 +1404,14 @@ class SoundFile(object):
             raise ValueError("Invalid shape: {0!r}".format(array.shape))
         if not array.flags.c_contiguous:
             raise ValueError("Data must be C-contiguous")
-        try:
-            ctype = _ffi_types[array.dtype.name]
-        except KeyError:
-            raise TypeError("dtype must be one of {0!r}".format(
-                sorted(_ffi_types.keys())))
+        ctype = self._check_dtype(array.dtype.name)
         assert array.dtype.itemsize == _ffi.sizeof(ctype)
         cdata = _ffi.cast(ctype + '*', array.__array_interface__['data'][0])
         return self._cdata_io(action, cdata, ctype, frames)
 
     def _cdata_io(self, action, data, ctype, frames):
         """Call one of libsndfile's read/write functions."""
-        if ctype not in _ffi_types.values():
-            raise ValueError("Unsupported data type: {0!r}".format(ctype))
+        assert ctype in _ffi_types.values()
         self._check_if_closed()
         if self.seekable():
             curr = self.tell()
@@ -1410,7 +1452,7 @@ def _error_check(err, prefix=""):
     """Pretty-print a numerical error code if there is an error."""
     if err != 0:
         err_str = _snd.sf_error_number(err)
-        raise RuntimeError(prefix + _ffi.string(err_str).decode())
+        raise RuntimeError(prefix + _ffi.string(err_str).decode('utf-8', 'replace'))
 
 
 def _format_int(format, subtype, endian):
@@ -1506,7 +1548,7 @@ def _get_format_from_filename(file, mode):
         # This raises an exception if file is not a (Unicode/byte) string:
         format = _os.path.splitext(file)[-1][1:]
         # Convert bytes to unicode (raises AttributeError on Python 3 str):
-        format = format.decode()
+        format = format.decode('utf-8', 'replace')
     except Exception:
         pass
     if format.upper() not in _formats and 'r' not in mode:
@@ -1521,6 +1563,8 @@ def _format_str(format_int):
         for k, v in dictionary.items():
             if v == format_int:
                 return k
+    else:
+        return 'n/a'
 
 
 def _format_info(format_int, format_flag=_snd.SFC_GET_FORMAT_INFO):
@@ -1531,7 +1575,7 @@ def _format_info(format_int, format_flag=_snd.SFC_GET_FORMAT_INFO):
                     _ffi.sizeof("SF_FORMAT_INFO"))
     name = format_info.name
     return (_format_str(format_info.format),
-            _ffi.string(name).decode() if name else "")
+            _ffi.string(name).decode('utf-8', 'replace') if name else "")
 
 
 def _available_formats_helper(count_flag, format_flag):
